;------------------------------------------------------------------------------
; COLOUR BITMAP
; Code Copyright (C)2003 Andrew Davie -- atari2600@taswegian.com
; May be freely used for private, educational and research purposes.
; If you wish to profit from it, please ask me for permission first.
; Distributed to the [stella] list 12/February/2003


                processor 6502

VSYNC           equ $40                 ;   0000 00x0       Vertical Sync Set-Clear
VBLANK          equ $41                 ;   xx00 00x0       Vertical Blank Set-Clear
WSYNC           equ $42                 ;   ---- ----       Wait for Horizontal Blank
COLUPF          equ $48                 ;   xxxx xxx0       Color-Luminance Playfield
COLUBK          equ $49                 ;   xxxx xxx0       Color-Luminance Background
CTRLPF          equ $4A                 ;   00xx 0xxx       Control Playfield, Ball, Collisions
PF0             equ $4D                 ;   xxxx 0000       Playfield Register Byte 0
PF1             equ $4E                 ;   xxxx xxxx       Playfield Register Byte 1
PF2             equ $4F                 ;   xxxx xxxx       Playfield Register Byte 2
INTIM           equ $284                ;                   Timer output
TIM64T          equ $296                ;                   set 64 clock interval

;------------------------------------------------------------------------------

                SEG.U variables
                ORG $80

LineR0          ds 2                    ; 6 columns of RED sub-image
LineR1          ds 2
LineR2          ds 2
LineR3          ds 2
LineR4          ds 2
LineR5          ds 2

LineG0          ds 2                    ; 6 columns of GREEN sub-image
LineG1          ds 2
LineG2          ds 2
LineG3          ds 2
LineG4          ds 2
LineG5          ds 2

LineB0          ds 2                    ; 6 columns of BLUE sub-image
LineB1          ds 2
LineB2          ds 2
LineB3          ds 2
LineB4          ds 2
LineB5          ds 2

Phase           ds 1

;------------------------------------------------------------------------------

            SEG code
            ORG $F000

    ; The image file is composed of strips (corresponding to PF0-PF2, repeated)
    ; which are decomposed by a utility from colour-separated RGB components of
    ; an original image. The displayed image is 40 pixels x 192 pixels.

    ; For access to the conversion/decomposition tool which generated these data
    ; please contact me privately.

        include "testblue.asm"
        include "testred.asm"
        include "testgreen.asm"

;------------------------------------------------------------------------------


Reset
            sei
            cld
               
		    ldx #$FF
		    txs 				; Clear the stack


    ; Start of vertical blank processing


NewFrame    lda #2
            sta WSYNC

            sta VBLANK          

            sta VSYNC           
            sta WSYNC           
            sta WSYNC
            sta WSYNC

            lda #0
            sta VSYNC           

            lda #43             
            sta TIM64T

            lda #0
            sta COLUBK

VblankLoop  lda INTIM
            bne VblankLoop      
            
            sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #238                ; NTSC
            sta TIM64T

            lda #%00010100      ; NO reflect, priority, 2 pixel wide ball
            sta CTRLPF
          
            lda #0
            sta PF0
            sta PF1
            sta PF2

    ;--------------------------------------------------------------------------
    ; START OF COLOUR BITMAP DISPLAY KERNEL


RED         = %00110100
GREEN       = $D6
BLUE        = %01110000


            lda #<testred_STRIP_0
            sta LineR0
            lda #>testred_STRIP_0
            sta LineR0+1

            lda #<testred_STRIP_1
            sta LineR1
            lda #>testred_STRIP_1
            sta LineR1+1

            lda #<testred_STRIP_2
            sta LineR2
            lda #>testred_STRIP_2
            sta LineR2+1

            lda #<testred_STRIP_3
            sta LineR3
            lda #>testred_STRIP_3
            sta LineR3+1

            lda #<testred_STRIP_4
            sta LineR4
            lda #>testred_STRIP_4
            sta LineR4+1

            lda #<testred_STRIP_5
            sta LineR5
            lda #>testred_STRIP_5
            sta LineR5+1



            lda #<testgreen_STRIP_0
            sta LineG0
            lda #>testgreen_STRIP_0
            sta LineG0+1

            lda #<testgreen_STRIP_1
            sta LineG1
            lda #>testgreen_STRIP_1
            sta LineG1+1

            lda #<testgreen_STRIP_2
            sta LineG2
            lda #>testgreen_STRIP_2
            sta LineG2+1

            lda #<testgreen_STRIP_3
            sta LineG3
            lda #>testgreen_STRIP_3
            sta LineG3+1

            lda #<testgreen_STRIP_4
            sta LineG4
            lda #>testgreen_STRIP_4
            sta LineG4+1

            lda #<testgreen_STRIP_5
            sta LineG5
            lda #>testgreen_STRIP_5
            sta LineG5+1


            lda #<testblue_STRIP_0
            sta LineB0
            lda #>testblue_STRIP_0
            sta LineB0+1

            lda #<testblue_STRIP_1
            sta LineB1
            lda #>testblue_STRIP_1
            sta LineB1+1

            lda #<testblue_STRIP_2
            sta LineB2
            lda #>testblue_STRIP_2
            sta LineB2+1

            lda #<testblue_STRIP_3
            sta LineB3
            lda #>testblue_STRIP_3
            sta LineB3+1

            lda #<testblue_STRIP_4
            sta LineB4
            lda #>testblue_STRIP_4
            sta LineB4+1

            lda #<testblue_STRIP_5
            sta LineB5
            lda #>testblue_STRIP_5
            sta LineB5+1

            dec Phase
            bpl PhaseOK
            lda #2
            sta Phase
PhaseOK

            
            ldy #191                ; total number of scanlines to display


    ; Vector to the correct start - a red, blue or green line

            ldx Phase
            beq ScanRed
            dex
            beq ScanGreen
            bne ScanBlue

ScanRed     sta WSYNC

            lda #RED
            sta COLUPF

            lda (LineR0),y
            sta PF0
            lda (LineR1),y
            sta PF1
            lda (LineR2),y
            sta PF2
            lda (LineR3),y
            sta PF0
            lda (LineR4),y
            sta PF1
            lda (LineR5),y
            sta PF2

            dey
            beq ScanEnd

          
ScanGreen   STA WSYNC

            lda #GREEN
            sta COLUPF

            lda (LineG0),y
            sta PF0
            lda (LineG1),y
            sta PF1
            lda (LineG2),y
            sta PF2
            lda (LineG3),y
            sta PF0
            lda (LineG4),y
            sta PF1
            lda (LineG5),y
            sta PF2

            dey
            beq ScanEnd


ScanBlue    sta WSYNC
                
            lda #BLUE
            sta COLUPF

            lda (LineB0),y
            sta PF0
            lda (LineB1),y
            sta PF1
            lda (LineB2),y
            sta PF2
            lda (LineB3),y
            sta PF0
            lda (LineB4),y
            sta PF1
            lda (LineB5),y
            sta PF2

            dey
            bne ScanRed

ScanEnd

    ; Screen has been drawn - now just tidy up

            sty WSYNC
            sty PF0
            sty PF1
            sty PF2

PadScreen2  lda INTIM
            bne PadScreen2
  
            sta WSYNC

            lda #%01000010
            sta VBLANK                      ; end of screen - enter blanking

            lda #25
            sta TIM64T

Overscan    lda INTIM
            bne Overscan

            sta WSYNC           
            sta WSYNC           

            jmp NewFrame

    ;--------------------------------------------------------------------------

    ORG $fffa 

            .word Reset           ; NMI
            .word Reset           ; RESET
            .word Reset           ; IRQ


